/* ---------------------------------------------------------------------------
 * rcespp (Resource contrained elementary shortest path problem)
 * ---------------------------------------------------------------------------
 * Author: Pedro Munari (pedro.munari@gmail.com)
 * University of Sao Paulo, Brazil
 * Last update: Feb/2011
 * ---------------------------------------------------------------------------
 */

#ifndef MYLIB_RCESPP_STATES
#define MYLIB_RCESPP_STATES

typedef char T_byte;

/* Data structure to represent a state (label) */
typedef struct RCESPP_state_type_str
{
    /* INFORMATION ABOUT THE PATH */
    double cost;	   /* total cost of the path*/
    double cost_0;	   /* the total cost without the cut prizes */
    
    double rem_prizes;
    
    int    time;       /* RESOURCE: Elapsed time of the path */
    int    capacity;   /* RESOURCE: Consumed capacity of the path */
    T_byte *visited;   /* RESOURCE: Dummy (nodes visited by the path) */
    int    *SRC_set;   /* RESOURCE: Number of nodes in the path that belongs to a set S_q 
    					  of the SRC cuts */
    
    int  length;	   /* length of the path */
    int  vertex;	   /* index of the associated vertex */
    int  status;	   /* flag: 0 -> inactive; 1 -> active, not extended; 2 -> active, 
    					  extended; 3 -> joined */
	int  version;
	
	/* Navigation pointers for the LIST of states of a vertex */    
    struct RCESPP_state_type_str *next_state;
    struct RCESPP_state_type_str *previous_state; 

	/* Navigation pointers for the TREE of states inheritance */
    struct RCESPP_state_type_str *parent_state;    
    struct RCESPP_state_type_str *last_child;
    struct RCESPP_state_type_str *next_sibling;
}
RCESPP_state_type;

/* Data structure to store the forward and backward lists */
typedef struct
{
	/* Number of vertices (including source and sink) */ 
	int N;
	
	/* Pointers to the list of forward states of each node */
    RCESPP_state_type **states_fw;
    
	/* Pointers to the list of backward states of each node */
    RCESPP_state_type **states_bw;
    
    /* Best cost value of each states list */
    double *best_cost_fw;
    double *best_cost_bw;
}
RCESPP_states_lists_type;


#endif

